/* This file is part of Error Correctiong Codes Application.
 *
 * Ecc-Application is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Ecc-Application is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Ecc-Application; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */
 

package ch.educeth.eccapplet; 

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;


/**
 * Setzt die Modell- und die Viewklassen zusammen.
 */
public final class EccController extends JPanel {
	/** Nur eine Instanz, implementiert Singelton Pattern */
	protected static final EccController CONTROLLER = new EccController();
	/* weitere Initialisation, die aber nicht im Konstrukor stehen 
	 * kann, da sich dieser sonst aufruft bevor es initialisiert ist */
	static {
		CONTROLLER.empi.setPredefinedPicture(1);
		CONTROLLER.empi.setErrorPicture();
	}
	/** Modell der Fehlerwahrscheinlichkeit */
	protected EccModelErrorProbability emep;
	/** Modell der Farben */
	protected EccColorModel ecmo;
	/** Modell der Bilder */
	protected EccModelPicture empi;
	/** GUI der Applikation */
	protected EccView ev;
	private int paintColor = 1;
	
	/** Konstruktor, haengt Observer und Observable zusammen; 
	 * plus Eventbehandlung
	 */
	private EccController() {
		super();
		emep = new EccModelErrorProbability();
		ecmo = new EccColorModel();
		empi = new EccModelPicture();
		ev = new EccView();
		emep.addObserver(ev);
		emep.setErrorProbability(0);
		empi.addObserver(ev);
		ecmo.addObserver(ev);
		this.setLayout(new BorderLayout());
		this.add(ev.mainPanel);

		ev.addButtonListener(new ActionListener() {
			public void actionPerformed(ActionEvent a) {
				int errorProb = emep.getErrorProbability();
				emep.setErrorProbability(errorProb);
			}
		});
	
		ev.addButton1Listener(new ActionListener() {
			public void actionPerformed(ActionEvent a) {
				paintColor = 1;
				ev.setIcons(1);
			}
		});
	
		ev.addButton2Listener(new ActionListener() {
			public void actionPerformed(ActionEvent a) {
				paintColor = 2;	
				ev.setIcons(2);
			}
		});
	
		ev.addButton3Listener(new ActionListener() {
			public void actionPerformed(ActionEvent a) {
				paintColor = 3;
				ev.setIcons(3);
			}
		});
	
		ev.addButton4Listener(new ActionListener() {
			public void actionPerformed(ActionEvent a) {
				paintColor = 4;
				ev.setIcons(4);
			}
		});

		ev.addSliderListener(new ChangeListener() {
			public void stateChanged(ChangeEvent e) {
				int slInt = 0;
				JSlider source = (JSlider)e.getSource();
				slInt = (int)source.getValue();
				emep.setErrorProbability(slInt);
			}
		});

		ev.addItemListener(new ItemListener() {
			public void itemStateChanged(ItemEvent e) {
				empi.setPredefinedPicture(1+ev.topLeftComboBox.getSelectedIndex());
			}
		});

		ev.addItem2Listener(new ItemListener() {
			public void itemStateChanged(ItemEvent e) {
				int temp = ev.middleLeftComboBox.getSelectedIndex();
				if (temp == 5) {
					ecmo.setNumOfDim(13);
				} else {
					ecmo.setNumOfDim(temp + 3);
				}
				ecmo.setGeneratorFlag(true);
				ecmo.setModifiedFlag(false);
			}
		});
		
		ev.addMouseListener(new MouseAdapter() {
			public void mousePressed(MouseEvent e) {
				if (String.valueOf(ev.topLeftComboBox.getSelectedItem()).equals(EccConstants.PIC_OWN)) {
					int x = ev.getPicX();
					int y = ev.getPicY();
					int i, u, transx, transy;
					int xPos = 0, yPos = 0;
					int xx = EccConstants.X5;
					int yy = EccConstants.Y5;
					i = (int)((float)(x - 20)/(float)xx);
					u = (int)((float)(y - 30)/(float)yy);	
					if (u < i) {
						i = u;
					}
					transx = (x - i*xx)/2;
					transy = (y - i*yy)/2;
					if ((e.getX() > transx) && (e.getX() < transx + xx*i) &&
						 (e.getY() > transy) && (e.getY() < transy + yy*i)) {
						xPos = ((e.getX() - transx)/i); 
						yPos = ((e.getY() - transy)/i);
						empi.setPicturePoint(yPos, xPos, paintColor);
					}
				}
			}
		});

		ev.rightPanel2.addMathButtonListener(new ActionListener() {
			public void actionPerformed(ActionEvent a) {
				int dim = ecmo.getNumOfDim();
				EccGeneratorFrame egf = new EccGeneratorFrame(dim);
			}
		});

	}

}	
