/* This file is part of Error Correctiong Codes Application.
 *
 * Ecc-Application is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Ecc-Application is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Ecc-Application; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */


package ch.educeth.eccapplet;

import java.util.Observable;
import java.awt.*;


/**
 * Diese Klasse enthaelt das Modell fuer die verschiedenen Bilder.
 * Aendern sich diese, werden die entsprechenden Observer benachrichtigt.
 */
public class EccModelPicture extends Observable {
	private int picXsize;
	private int picYsize;
	private int mainPicture[][];
	private int errorPicture[][];
	private Color decodedPicture[][];

	/** Konstruktor, initialisiert alle Bilder */
	public EccModelPicture() {
		mainPicture = new int[EccConstants.MAXHEIGHT][EccConstants.MAXWIDTH];
		errorPicture = new int[EccConstants.MAXHEIGHT][EccConstants.MAXWIDTH];
		decodedPicture = new Color[EccConstants.MAXHEIGHT][EccConstants.MAXWIDTH];
	}

	/**
	 * Setzt die Groesse des Bildes in x-Richtung
	 * @param value Groesse des Bildes in x-Richtung
	 */
	public void setXsize(int value) {
		picXsize = value;
		setChanged();
		notifyObservers();
	}	/* setXsize */

	/**
	 * Liefert die Groesse des Bildes in x-Richtung
	 * @return Groesse des Bildes in x-Richtung
	 */
	public int getXsize() {
		return picXsize;
	}	/* getXsize */

	/**
	 * Setzt die Groesse des Bildes in y-Richtung
	 * @param Groesse des Bildes in x-Richtung
	 */
	public void setYsize(int value) {
		picYsize = value;
		setChanged();
		notifyObservers();
	}	/* setYsize */

	/**
	 * Liefert die Groesse des Bildes in y-Richtung
	 * @return Groesse des Bildes in y-Richtung
	 */
	public int getYsize() {
		return picYsize;
	}	/* getYsize */

	/**
	 * Setzt den Wert eines Punktes - beim Bild, welches bearbeitet werden kann
	 * @param x x-Koordinate des Punktes
	 * @param y y-Koordinate des Punktes
	 * @param color Wert des Punktes
	 */
	public void setPicturePoint(int x, int y, int color) {
		mainPicture[x][y] = color;
		setChanged();
		notifyObservers();
	}	/* setPicturePoint */

	/**
	 * Liefert den Wert eines Punktes
	 * @param x x-Koordinate des Punktes
	 * @param y y-Koordinate des Punktes
	 * @return Wert des Punktes
	 */
	public int getPicturePoint(int x, int y) {
		return mainPicture[x][y];
	}	/* getPicturePoint */

	/**
	 * Liefert den Wert eines fehlerbehafteten Punktes
	 * @param x x-Koordinate des Punktes
	 * @param y y-Koordinate des Punktes
	 * @return Wert des fehlerbehafteten Punktes
	 */
	public int getErrorPicturePoint(int x, int y) {
		return errorPicture[x][y];
	}	/* getErrorPicturePoint */

	/**
	 * Liefert den Wert eines decodierten Punktes
	 * @param x x-Koordinate des Punktes
	 * @param y y-Koordinate des Punktes
	 * @return Wert des decodierten Punktes
	 */
	public Color getDecodedPicturePoint(int x, int y) {
		return decodedPicture[x][y];
	}	/* getDecodedPicturePoint */
	
	/**
	 * Liefert die Anzahl der Fehler im decodierten Bild
	 * @return Anzahl der Fehler
	 */
	public int numberOfErrors() {
		int sum = 0;
		for (int i = 0; i < picYsize; i++) {
			for (int j = 0; j < picXsize; j++) {
				if (!EccController.CONTROLLER.ecmo.getOriginalColor(mainPicture[i][j]).equals(decodedPicture[i][j])) {
					sum++;
				}
			}
		}
		return sum;
	}	/* numberOfErrors */

	/**
	 * Setzt eines der vorgegebenen Bilder
	 * @param value Bildnummer
	 */
	public void setPredefinedPicture(int value) {
		switch (value)
		{
			case 1:
				{
					EccController.CONTROLLER.ev.removeButtons();
					picXsize = EccConstants.X1;
					picYsize = EccConstants.Y1;
					for(int i=0; i < picYsize; i++){
						for(int j=0; j < picXsize; j++){
							mainPicture[i][j] = EccConstants.PICTURE1[i][j];
						}
					}
					break;
				}
			case 2:
				{
					EccController.CONTROLLER.ev.removeButtons();
					picXsize = EccConstants.X2;
					picYsize = EccConstants.Y2;
					for(int i=0; i < picYsize; i++){
						for(int j=0; j < picXsize; j++){
							mainPicture[i][j] = EccConstants.PICTURE2[i][j];
						}
					}
					break;
				}
			case 3:
				{
					EccController.CONTROLLER.ev.removeButtons();
					picXsize = EccConstants.X3;
					picYsize = EccConstants.Y3;
					for(int i=0; i < picYsize; i++){
						for(int j=0; j < picXsize; j++){
							mainPicture[i][j] = EccConstants.PICTURE3[i][j];
						}
					}
					break;
				}
			case 4:
				{
					EccController.CONTROLLER.ev.removeButtons();
					picXsize = EccConstants.X4;
					picYsize = EccConstants.Y4;
					for(int i=0; i < picYsize; i++){
						for(int j=0; j < picXsize; j++){
							mainPicture[i][j] = EccConstants.PICTURE4[i][j];
						}
					}
					break;
				}
			case 5:
				{
					picXsize = EccConstants.X5;
					picYsize =EccConstants.Y5;
					for(int i=0; i < picYsize; i++){
						for(int j=0; j < picXsize; j++){
							mainPicture[i][j] = EccConstants.PICTURE5[i][j];
						}
					}
					EccController.CONTROLLER.ev.addButtons();
					break;
				}
			default:
				{
					System.err.println("This picture number does not exist");
				}
		}	/* end of switch */
		setChanged();
		notifyObservers();
	}	/* setPredefinedPicture */

	/**
	 * Update des Fehlerbildes, weil sich zum Beispiel die
	 * Fehlerwahrscheinlichkeit geaendert hat
	 */
	public void setErrorPicture() {
		int errorProb, number, dim;
		dim = EccController.CONTROLLER.ecmo.getNumOfDim();
		errorProb = EccController.CONTROLLER.emep.getErrorProbability();
		for (int i=0; i < picYsize; i++) {
			for (int j=0; j < picXsize; j++) {
				int[] bin = new int[dim];
				int row = mainPicture[i][j] - 1;
				for (int k=0; k < dim; k++) {
					bin[k] = EccController.CONTROLLER.ecmo.getMatrixAt(row, k);
				}
				for (int l=0; l < dim; l++) {
					if (errorProb > Math.random()*100) { 
						bin[l] = (bin[l] + 1) % 2;
					}
				}
				errorPicture[i][j] = EccColorModel.getNumber(bin, dim);
			}
		}
	}	/* setErrorPicture */

	/**
	 * Berechnet das decodierte Bild
	 */
	public void setDecodedPicture() {
		int temp;
		for (int i=0; i< picYsize; i++) {
			for (int j=0; j< picXsize; j++) {
				temp = this.getErrorPicturePoint(i, j);
				int dim = EccController.CONTROLLER.ecmo.getNumOfDim();				
				Color col1 = new Color(0,0,0);
				int[] binary = new int[dim];
				EccController.CONTROLLER.ecmo.getBinary(temp, binary, dim);
				col1 = EccController.CONTROLLER.ecmo.getNearestColor(binary, dim);
				decodedPicture[i][j] = col1;
			}
		}
	}	/* setDecodedPicture */

}
	

