/* This file is part of Error Correctiong Codes Application.
 *
 * Ecc-Application is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Ecc-Application is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Ecc-Application; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */


/*
 *  ----------------------------------------------------------------------------
 *  Filename      : LightweightDragDrop.java
 *  ----------------------------------------------------------------------------
 *  Application   : [ k2j ]
 *  Author        : Thomas Suter (original version ch.ethz.dndpanel),
 *                  Raimond Reichert (this revised version)
 *  File created  : August 2001
 *  ----------------------------------------------------------------------------
 */


package ch.educeth.util.gui.dndpanel;

import java.awt.image.*;
import java.awt.*;


/**
 * LeightweightDragDrop defines an interface for visual components
 * which enables them to transfer Objects. In contrast to the Java Drag and Drop
 * mechanism this interface provides some simple callback methods to check for
 * acceptance of drag and drop as well as for transfering the desired objects.
 * Components implementing this interface must have a parent of basetype
 * <b>DNDpanel</b> which handels the transfer of the objects.
 */
public interface LightweightDragDrop {

  /**
   * Checks if the component is willing to accept from the position in event.
   * If it does, acceptsDrag is to return the data object to be transfered,
   * otherwise, null.
   */
  public Object acceptsDrag (java.awt.event.MouseEvent event);

  /**
   * Checks if the component is willing to receive dropping of obj at the
   * position of the event.
   */
  public boolean acceptsDrop (java.awt.event.MouseEvent event, Object obj);

  /**
   * Dropps the object into the target component at the position of the event.
   * This method will only be called if acceptsDrop returned true for
   * the same parameter values.
   */
  public void dropTransferObject (java.awt.event.MouseEvent event, Object obj);

  /**
   * getDragImage provides an image represenation of the dragObject
   * which is to dragged. This method will only be called if acceptsDrag
   * returned true.
   */
   public BufferedImage getDragImage (Object dragObject);

}
